dojo.provide('ibm_ilog.diagram.interactors.AdornerFocusInteractor');

dojo.require('ibm_ilog.diagram.interactors.Interactor');
dojo.require('ibm_ilog.diagram.util.Geometry');
dojo.require('ibm_ilog.diagram.util.GraphUtil');

(function () {
	var g = ibm_ilog.diagram.util.Geometry;
	var gu = ibm_ilog.diagram.util.GraphUtil;

dojo.declare("ibm_ilog.diagram.interactors.AdornerFocusInteractor", [ibm_ilog.diagram.interactors.Interactor], {
	//
	//	summary:
	//		This interactor sets the diagram widget adorners focus 
	//		according to current mouse location. 
	//
	
	//
	//	_viewport: ibm_ilog.diagram.widget.Viewport
	//		The diagram viewport 
	//
	_viewport: null,

	//
	//	_diagram: ibm_ilog.diagram.widget.Diagram
	//
	_diagram: null,

	_nodeMargin: 15,
	_linkMargin: 15,

	_lowCpuMode: null,

	_lastPosition: null,
	
	_selectionChangedSinceLastDetectOnOver: false,

	initialize: function ( /*ibm_ilog.diagram.widget.Diagram*/ diagram) {
		this._diagram = diagram;
		this._viewport = diagram.getViewport();
		this._initialize();
		this._lowCpuMode = this._diagram.isLowCpuMode();
		
		dojo.connect(this._diagram, 'lowCpuModeChange', this, function () {
			this._lowCpuMode = this._diagram.isLowCpuMode();
			this.deactivate();
			this._initialize();
		});
		
		dojo.connect(this._diagram.getSelection(), "onSelectionChanged", this, "_onSelectionChanged");
		
		this._lastPosition = {
			x: 0,
			y: 0
		};
		return this;
	},

	_declareStates: function () {
		if (this._lowCpuMode == true) {
			this._declareState("idle", ["nodeover", "linkover", "subgraphover", "testfreecursor","onLeaveSurface"]);
		} else {
			this._declareState("idle", ["detectOnOver","onLeaveSurface"]);
		}
	},

	getDefaultConnections: function () {
		//
		//	summary:
		//		returns the default event mapping
		//
		return {
			nodeover: {
				src: this._diagram,
				srcEvt: "onNodeMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			linkover: {
				src: this._diagram,
				srcEvt: "onLinkMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			subgraphover: {
				src: this._diagram,
				srcEvt: "onSubgraphMouseOver",
				connectTo: "_setNodeAdornersFocus"
			},
			testfreecursor: {
				gfxConnect: true,
				src: this._viewport.getEventSource(),
				srcEvt: "onmousemove",
				connectTo: "_testFreeCursor"
			},
			detectOnOver: {
				src: this._viewport.getEventSource(),
				srcEvt: "onmousemove",
				connectTo: "_detectOnOver",
				gfxConnect: true
			},
			onLeaveSurface: {
				src: this._diagram.domNode,
				srcEvt: "onmouseleave",
				connectTo: "_onLeaveSurface"
			}
		};
	},
	
	_onLeaveSurface: function(e) {
		// Check if we are completely outside of the surface,
		// in that case we get no move event so we must free the adorners.
		var out = false;
		var l = this._viewport.eventClientLocation(e);
		if(l.x <= 0 || l.y <= 0){
			out = true;
		} else {
			var d = this._viewport.getClientSize();
			if(l.x >= d.width || l.y >= d.height)
				out = true;
		}
		if(out){
			this._freeCursor();
			this._freeLinkCursor();
		}
	},

	_setNodeAdornersFocus: function (ge, e) {
		// Summary:
		//		Sets the appropriate cursor, according to the ge element. Release the previous cursor, id applied.
		//		
		var cursorIsFree = this._diagram.getNodeAdornersFocus() == null;
		if (!ge._isIBMDiagramLink && (cursorIsFree || gu.isContainedBy(ge, this._diagram.getNodeAdornersFocus()))) {
			this._freeLinkCursor(null);
			this._diagram.setNodeAdornersFocus(ge);
			dojo.stopEvent(e);
		} else if (ge._isIBMDiagramLink) {
			this._freeCursor(null);
			this._diagram.setLinkAdornersFocus(ge);
			dojo.stopEvent(e);
		}
	},

	onEnabled: function () {
		this._goStateId("idle");
	},

	onDisabled: function () {
		this._goState(null);
		this._freeCursor();
	},

	_freeCursor: function (e) {
		// Summary:
		//		Free the node/subgraph cursor
		//
		this._diagram.setNodeAdornersFocus(null);
	},

	_freeLinkCursor: function (e) {
		// Summary:
		//		Free the link cursor
		//
		this._diagram.setLinkAdornersFocus(null);
	},

	_testFreeCursor: function (e) {
		// Summary:
		//		Free the cursor testing the bounding box (adding a margin)
		//
		var D = this._viewport.getDiagram();
		var c = this._diagram.getNodeAdornersFocus();
		if (c) {
			var outOfBounds;
			var bb = g.expandRect(c.getBounds(this._viewport.getSurface()), this._nodeMargin);
			var l = this._viewport.eventClientLocation(e);
			outOfBounds = !g.containsPoint(bb, l);
			if (D.mustFreeAdornersFocus(c, p, this._nodeMargin, outOfBounds)) {
				this._freeCursor();
			}
		}
		c = this._diagram.getLinkAdornersFocus();
		if (c) {
			var V = this._viewport;
			var p = V.eventContentLocation(e);
			outOfBounds = !c.hitTest(p, D.getGraph(), this._linkMargin);
			if (D.mustFreeAdornersFocus(c, p, this._linkMargin, outOfBounds)) {
				this._freeLinkCursor();
			}
		}
	},

	_onSelectionChanged: function (e) {		
		// AVA: added for fixing defect 2111
		this._selectionChangedSinceLastDetectOnOver = true;
	},
	
	_detectOnOver: function (e) {
		//added to increase performance
		var current = {
			x: e.pageX,
			y: e.pageY
		};
		var delta = {
			x: Math.abs(current.x - this._lastPosition.x),
			y: Math.abs(current.y - this._lastPosition.y)
		};
		var minDelta = 2;
		// AVA: take into account the change of selection which
		// can lead to a change of focus even if the mouse did not 
		// move (fix of Defect 2111)
		if (!this._selectionChangedSinceLastDetectOnOver &&
				(delta.x < minDelta && delta.y < minDelta)) {			
			return;
		}
		this._selectionChangedSinceLastDetectOnOver = false;
		this._testFreeCursor(e);
		var p = this._viewport.eventContentLocation(e);
		var ges = this._diagram.getGraph().hitTest(p, 4);
		var ge = this._changeFocus(ges);
		if (ge) {
			this._setNodeAdornersFocus(ge, e);
		}
		this._lastPosition = {
			x: current.x,
			y: current.y
		};

	},
	_changeFocus: function (ges) {
		if (ges.length > 0) {
			var D = this._viewport.getDiagram();
			var anyCursor = this._diagram.getNodeAdornersFocus() || this._diagram.getLinkAdornersFocus();
			var changeAdornersFocus = true;
			if (anyCursor && (anyCursor.isSelected() || anyCursor._isIBMDiagramLink)) {
				changeAdornersFocus = false;
			}

			var newge = ges[0];
			for (var index in ges) {
				if (ges[index].isSelected()) {
					newge = ges[index];
					break;
				}
			}
			/*
			var newge = null;
			for(var index in ges){
				if(ges[index].isSelected()){
					newge = ges[index];
					break;
				}else{
					if(!newge && ges[index]._isIBMDiagramNode){
						newge = ges[index];
					}
				}
			}
			newge = newge?newge:ges[0];
			*/
			if (newge._isIBMDiagramLink && !newge.isSelected()) {
				if (anyCursor && anyCursor._isIBMDiagramNode && !gu.isContainedBy(newge, anyCursor)) {
					changeAdornersFocus = false;
				}
			}

			return D.changeAdornersFocus(anyCursor, newge, ges, changeAdornersFocus);
		}
		return null;
	}
});

})();
